@@ -1,48 +1,26 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/*************************************************************************
- *
- * Copyright (c) 2020 Kohei Yoshida
- *
- * Permission is hereby granted, free of charge, to any person
- * obtaining a copy of this software and associated documentation
- * files (the "Software"), to deal in the Software without
- * restriction, including without limitation the rights to use,
- * copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following
- * conditions:
- *
- * The above copyright notice and this permission notice shall be
- * included in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
- * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
- * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
- * OTHER DEALINGS IN THE SOFTWARE.
- *
- ************************************************************************/
+
+// SPDX-FileCopyrightText: 2020 - 2025 Kohei Yoshida
+//
+// SPDX-License-Identifier: MIT
 
 #include <iostream>
 #include <vector>
 #include <fstream>
 #include <mdds/trie_map.hpp>
 
-using std::cout;
-using std::endl;
-
 int main() try
 {
-    using map_type = mdds::packed_trie_map<mdds::trie::std_string_trait, int>;
+    //!code-start: type
+    using map_type = mdds::packed_trie_map<std::string, int>;
+    //!code-end: type
 
     // List of world's largest cities and their populations. The entries must
     // be sorted by the keys.
     //
     // c.f. https://en.wikipedia.org/wiki/List_of_largest_cities#cities
 
+    //!code-start: entries
     std::vector<map_type::entry> entries =
     {
         { MDDS_ASCII("Ahmedabad"),        7681000  },
@@ -127,47 +105,63 @@ int main() try
         { MDDS_ASCII("Xi'an"),            7444000  },
         { MDDS_ASCII("Yangon"),           5157000  },
     };
+    //!code-end: entries
 
+    //!code-start: inst
     map_type cities(entries.data(), entries.size());
-    cout << "Number of cities: " << cities.size() << endl;
+    //!code-end: inst
 
-    cout << endl;
+    //!code-start: print-n-cities
+    std::cout << "Number of cities: " << cities.size() << std::endl;
+    //!code-end: print-n-cities
+
+    std::cout << std::endl;
 
     {
-        cout << "Cities that begin with 'S':" << endl;
+        //!code-start: search-s
+        std::cout << "Cities that begin with 'S':" << std::endl;
         auto results = cities.prefix_search("S");
         for (const auto& city : results)
-            cout << "  * " << city.first << ": " << city.second << endl;
+            std::cout << "  * " << city.first << ": " << city.second << std::endl;
+        //!code-end: search-s
     }
 
-    cout << endl;
+    std::cout << std::endl;
 
     {
+        //!code-start: save-state-cities
         std::ofstream outfile("cities.bin", std::ios::binary);
         cities.save_state(outfile);
+        //!code-end: save-state-cities
     }
 
+    //!code-start: load-state-cities
     map_type cities_loaded;
 
-    {
-        std::ifstream infile("cities.bin", std::ios::binary);
-        cities_loaded.load_state(infile);
-    }
+    std::ifstream infile("cities.bin", std::ios::binary);
+    cities_loaded.load_state(infile);
+    //!code-end: load-state-cities
 
-    std::ios_base::fmtflags origflags = cout.flags();
-    cout << "Equal to the original? " << std::boolalpha << (cities == cities_loaded) << endl;
-    cout.setf(origflags);
+    std::ios_base::fmtflags origflags = std::cout.flags();
+    //!code-start: check-equal
+    std::cout << "Equal to the original? " << std::boolalpha << (cities == cities_loaded) << std::endl;
+    //!code-end: check-equal
+    std::cout.setf(origflags);
 
-    cout << endl;
+    std::cout << std::endl;
 
-    cout << "Number of cities: " << cities_loaded.size() << endl;
+    //!code-start: print-n-cities-2
+    std::cout << "Number of cities: " << cities_loaded.size() << std::endl;
+    //!code-end: print-n-cities-2
 
-    cout << endl;
+    std::cout << std::endl;
 
-    cout << "Cities that begin with 'S':" << endl;
+    //!code-start: search-s-2
+    std::cout << "Cities that begin with 'S':" << std::endl;
     auto results = cities_loaded.prefix_search("S");
     for (const auto& city : results)
-        cout << "  * " << city.first << ": " << city.second << endl;
+        std::cout << "  * " << city.first << ": " << city.second << std::endl;
+    //!code-end: search-s-2
 
     return EXIT_SUCCESS;
 }
