@@ -1,42 +1,20 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-/*************************************************************************
- *
- * Copyright (c) 2008-2017 Kohei Yoshida
- *
- * Permission is hereby granted, free of charge, to any person
- * obtaining a copy of this software and associated documentation
- * files (the "Software"), to deal in the Software without
- * restriction, including without limitation the rights to use,
- * copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following
- * conditions:
- *
- * The above copyright notice and this permission notice shall be
- * included in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
- * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
- * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
- * OTHER DEALINGS IN THE SOFTWARE.
- *
- ************************************************************************/
 
-#ifndef INCLUDED_MDDS_FLAT_SEGMENT_TREE_HPP
-#define INCLUDED_MDDS_FLAT_SEGMENT_TREE_HPP
+// SPDX-FileCopyrightText: 2008 - 2025 Kohei Yoshida
+//
+// SPDX-License-Identifier: MIT
+
+#pragma once
 
 #include <iostream>
 #include <sstream>
 #include <utility>
 #include <cassert>
+#include <type_traits>
 
-#include "mdds/node.hpp"
-#include "mdds/flat_segment_tree_itr.hpp"
-#include "mdds/global.hpp"
+#include "./node.hpp"
+#include "./flat_segment_tree_itr.hpp"
+#include "./global.hpp"
 
 #ifdef MDDS_UNIT_TEST
 #include <cstdio>
@@ -55,144 +33,77 @@ public:
 
     struct nonleaf_value_type
     {
-        key_type low; /// low range value (inclusive)
-        key_type high; /// high range value (non-inclusive)
-
-        bool operator==(const nonleaf_value_type& r) const
+        bool operator==(const nonleaf_value_type&) const noexcept
         {
-            return low == r.low && high == r.high;
+            return true;
         }
-
-        nonleaf_value_type() : low(0), high(0)
-        {}
     };
 
     struct leaf_value_type
     {
-        key_type key;
         value_type value;
 
         bool operator==(const leaf_value_type& r) const
+            noexcept(noexcept(std::declval<value_type>() == std::declval<value_type>()))
         {
-            return key == r.key && value == r.value;
+            return value == r.value;
         }
 
-        leaf_value_type() : key(0), value()
-        {}
-    };
-
-    // Handlers required by the node template class.
-    struct fill_nonleaf_value_handler;
-    struct init_handler;
-    struct dispose_handler;
-#ifdef MDDS_UNIT_TEST
-    struct to_string_handler;
-#endif
-
-    typedef __st::node<flat_segment_tree> node;
-    typedef typename node::node_ptr node_ptr;
-
-    typedef __st::nonleaf_node<flat_segment_tree> nonleaf_node;
-
-    struct fill_nonleaf_value_handler
-    {
-        void operator()(
-            __st::nonleaf_node<flat_segment_tree>& _self, const __st::node_base* left_node,
-            const __st::node_base* right_node)
+        bool operator!=(const leaf_value_type& r) const noexcept(noexcept(!operator==(r)))
         {
-            // Parent node should carry the range of all of its child nodes.
-            if (left_node)
-            {
-                _self.value_nonleaf.low = left_node->is_leaf
-                                              ? static_cast<const node*>(left_node)->value_leaf.key
-                                              : static_cast<const nonleaf_node*>(left_node)->value_nonleaf.low;
-            }
-            else
-            {
-                // Having a left node is prerequisite.
-                throw general_error(
-                    "flat_segment_tree::fill_nonleaf_value_handler: Having a left node is prerequisite.");
-            }
-
-            if (right_node)
-            {
-                if (right_node->is_leaf)
-                {
-                    // When the child nodes are leaf nodes, the upper bound
-                    // must be the value of the node that comes after the
-                    // right leaf node (if such node exists).
-                    const node* p = static_cast<const node*>(right_node);
-                    if (p->next)
-                        _self.value_nonleaf.high = p->next->value_leaf.key;
-                    else
-                        _self.value_nonleaf.high = p->value_leaf.key;
-                }
-                else
-                {
-                    _self.value_nonleaf.high = static_cast<const nonleaf_node*>(right_node)->value_nonleaf.high;
-                }
-            }
-            else
-            {
-                _self.value_nonleaf.high = left_node->is_leaf
-                                               ? static_cast<const node*>(left_node)->value_leaf.key
-                                               : static_cast<const nonleaf_node*>(left_node)->value_nonleaf.high;
-            }
-        }
-    };
-
-#ifdef MDDS_UNIT_TEST
-    struct to_string_handler
-    {
-        std::string operator()(const node& _self) const
-        {
-            std::ostringstream os;
-            os << "(" << _self.value_leaf.key << ") ";
-            return os.str();
+            return !operator==(r);
         }
 
-        std::string operator()(const mdds::__st::nonleaf_node<flat_segment_tree>& _self) const
-        {
-            std::ostringstream os;
-            os << "(" << _self.value_nonleaf.low << "-" << _self.value_nonleaf.high << ") ";
-            return os.str();
-        }
-    };
-#endif
-
-    struct init_handler
-    {
-        void operator()(node& /*_self*/)
-        {}
-        void operator()(__st::nonleaf_node<flat_segment_tree>& /*_self*/)
+        leaf_value_type() : value{}
         {}
     };
 
-    struct dispose_handler
-    {
-        void operator()(node& /*_self*/)
-        {}
-        void operator()(__st::nonleaf_node<flat_segment_tree>& /*_self*/)
-        {}
-    };
+    using node = st::detail::node<key_type, leaf_value_type>;
+    using node_ptr = typename node::node_ptr;
+    using nonleaf_node = st::detail::nonleaf_node<key_type, nonleaf_value_type>;
 
 private:
-    friend struct ::mdds::__fst::itr_forward_handler<flat_segment_tree>;
-    friend struct ::mdds::__fst::itr_reverse_handler<flat_segment_tree>;
+    friend struct ::mdds::fst::detail::forward_itr_handler<flat_segment_tree>;
+    friend struct ::mdds::fst::detail::reverse_itr_handler<flat_segment_tree>;
+
+    static constexpr bool nothrow_move_constructible_v =
+        std::is_nothrow_move_constructible_v<std::vector<nonleaf_node>> &&
+        std::is_nothrow_move_constructible_v<value_type>;
+
+    static constexpr bool nothrow_swappable_v =
+        std::is_nothrow_swappable_v<value_type> && std::is_nothrow_swappable_v<std::vector<nonleaf_node>>;
+
+    static constexpr bool nothrow_move_assignable_v = nothrow_move_constructible_v && nothrow_swappable_v;
+
+    static constexpr bool nothrow_eq_comparable_v =
+        noexcept(std::declval<key_type>() == std::declval<key_type>()) &&
+        noexcept(std::declval<leaf_value_type>() == std::declval<leaf_value_type>());
 
 public:
-    class const_iterator : public ::mdds::__fst::const_iterator_base<
-                               flat_segment_tree, ::mdds::__fst::itr_forward_handler<flat_segment_tree>>
+    using const_segment_iterator = mdds::fst::detail::const_segment_iterator<flat_segment_tree>;
+
+    class const_iterator : public ::mdds::fst::detail::const_iterator_base<
+                               flat_segment_tree, ::mdds::fst::detail::forward_itr_handler<flat_segment_tree>>
     {
-        typedef ::mdds::__fst::const_iterator_base<
-            flat_segment_tree, ::mdds::__fst::itr_forward_handler<flat_segment_tree>>
+        typedef ::mdds::fst::detail::const_iterator_base<
+            flat_segment_tree, ::mdds::fst::detail::forward_itr_handler<flat_segment_tree>>
             base_type;
         friend class flat_segment_tree;
 
+        using base_type::get_parent;
+        using base_type::get_pos;
+        using base_type::is_end_pos;
+
     public:
         const_iterator() : base_type(nullptr, false)
         {}
 
+        /**
+         * Create a segment iterator that references the same segment the source
+         * iterator references the start key of.
+         */
+        const_segment_iterator to_segment() const;
+
     private:
         explicit const_iterator(const typename base_type::fst_type* _db, bool _end) : base_type(_db, _end)
         {}
@@ -201,11 +112,11 @@ public:
         {}
     };
 
-    class const_reverse_iterator : public ::mdds::__fst::const_iterator_base<
-                                       flat_segment_tree, ::mdds::__fst::itr_reverse_handler<flat_segment_tree>>
+    class const_reverse_iterator : public ::mdds::fst::detail::const_iterator_base<
+                                       flat_segment_tree, ::mdds::fst::detail::reverse_itr_handler<flat_segment_tree>>
     {
-        typedef ::mdds::__fst::const_iterator_base<
-            flat_segment_tree, ::mdds::__fst::itr_reverse_handler<flat_segment_tree>>
+        typedef ::mdds::fst::detail::const_iterator_base<
+            flat_segment_tree, ::mdds::fst::detail::reverse_itr_handler<flat_segment_tree>>
             base_type;
         friend class flat_segment_tree;
 
@@ -218,10 +129,20 @@ public:
         {}
     };
 
-    using const_segment_iterator = mdds::__fst::const_segment_iterator<flat_segment_tree>;
+    class const_segment_range_type
+    {
+        node_ptr m_left_leaf;
+        node_ptr m_right_leaf;
+
+    public:
+        const_segment_range_type(node_ptr left_leaf, node_ptr right_leaf);
+
+        const_segment_iterator begin() const;
+        const_segment_iterator end() const;
+    };
 
     /**
-     * Return an iterator that points to the first leaf node that correspondes
+     * Return an iterator that points to the first leaf node that corresponds
      * with the start position of the first segment.
      *
      * @return immutable iterator that points to the first leaf node that
@@ -246,7 +167,7 @@ public:
     }
 
     /**
-     * Return an iterator that points to the last leaf node that correspondes
+     * Return an iterator that points to the last leaf node that corresponds
      * with the end position of the last segment.  This iterator moves in the
      * reverse direction of a normal iterator.
      *
@@ -297,6 +218,13 @@ public:
      */
     const_segment_iterator end_segment() const;
 
+    /**
+     * Return a range object that provides a begin iterator and an end sentinel.
+     */
+    const_segment_range_type segment_range() const;
+
+    flat_segment_tree() = delete;
+
     /**
      * Constructor that takes minimum and maximum keys and the value to be
      * used for the initial segment.
@@ -313,21 +241,40 @@ public:
     /**
      * Copy constructor only copies the leaf nodes.
      */
-    flat_segment_tree(const flat_segment_tree<key_type, value_type>& r);
+    flat_segment_tree(const flat_segment_tree& r);
+
+    /**
+     * Move constructor.
+     *
+     * @warning The source instance will not be usable after the move
+     *          construction.
+     */
+    flat_segment_tree(flat_segment_tree&& other) noexcept(nothrow_move_constructible_v);
 
     ~flat_segment_tree();
 
     /**
-     * Assignment only copies the leaf nodes.
+     * Copy assignment operator.
+     *
+     * @param other Source instance to copy from.
+     *
+     * @note It only copies the leaf nodes.
      */
-    flat_segment_tree<key_type, value_type>& operator=(const flat_segment_tree<key_type, value_type>& other);
+    flat_segment_tree<Key, Value>& operator=(const flat_segment_tree& other);
+
+    /**
+     * Move assignment operator.
+     *
+     * @param other Source instance to move from.
+     */
+    flat_segment_tree<Key, Value>& operator=(flat_segment_tree&& other) noexcept(nothrow_move_assignable_v);
 
     /**
      * Swap the content of the tree with another instance.
      *
      * @param other instance of flat_segment_tree to swap content with.
      */
-    void swap(flat_segment_tree<key_type, value_type>& other);
+    void swap(flat_segment_tree& other) noexcept(nothrow_swappable_v);
 
     /**
      * Remove all stored segments except for the initial segment. The minimum
@@ -352,7 +299,7 @@ public:
      */
     std::pair<const_iterator, bool> insert_front(key_type start_key, key_type end_key, value_type val)
     {
-        return insert_segment_impl(start_key, end_key, val, true);
+        return insert_segment_impl(std::move(start_key), std::move(end_key), std::move(val), true);
     }
 
     /**
@@ -372,7 +319,7 @@ public:
      */
     std::pair<const_iterator, bool> insert_back(key_type start_key, key_type end_key, value_type val)
     {
-        return insert_segment_impl(start_key, end_key, val, false);
+        return insert_segment_impl(std::move(start_key), std::move(end_key), std::move(val), false);
     }
 
     /**
@@ -390,8 +337,7 @@ public:
      *         the inserted segment, and a boolean value indicating whether or
      *         not the insertion has modified the tree.
      */
-    std::pair<const_iterator, bool> insert(
-        const const_iterator& pos, key_type start_key, key_type end_key, value_type val);
+    std::pair<const_iterator, bool> insert(const_iterator pos, key_type start_key, key_type end_key, value_type val);
 
     /**
      * Remove a segment specified by the start and end key values, and shift
@@ -427,7 +373,7 @@ public:
      * @param start_key pointer to a variable where the start key value of the
      *                  segment that contains the key gets stored upon
      *                  successful search.
-     * @param end_key pointer to a varaible where the end key value of the
+     * @param end_key pointer to a variable where the end key value of the
      *                segment that contains the key gets stored upon
      *                successful search.
      * @return a pair of const_iterator corresponding to the start position of
@@ -443,13 +389,13 @@ public:
      *
      * @param pos position from which the search should start.  When the
      *            position is invalid, it falls back to the normal search.
-     * @param key key value
+     * @param key key value.
      * @param value value associated with key specified gets stored upon
      *              successful search.
      * @param start_key pointer to a variable where the start key value of the
      *                  segment that contains the key gets stored upon
      *                  successful search.
-     * @param end_key pointer to a varaible where the end key value of the
+     * @param end_key pointer to a variable where the end key value of the
      *                segment that contains the key gets stored upon
      *                successful search.
      * @return a pair of const_iterator corresponding to the start position of
@@ -457,9 +403,34 @@ public:
      *         whether or not the search has been successful.
      */
     std::pair<const_iterator, bool> search(
-        const const_iterator& pos, key_type key, value_type& value, key_type* start_key = nullptr,
+        const_iterator pos, key_type key, value_type& value, key_type* start_key = nullptr,
         key_type* end_key = nullptr) const;
 
+    /**
+     * Perform leaf-node search for a value associated with a key.
+     *
+     * @param key Key value to perform search for.
+     *
+     * @return Iterator position associated with the start position of the
+     *         segment containing the key, or end iterator position upon search
+     *         failure.
+     */
+    const_iterator search(key_type key) const;
+
+    /**
+     * Perform leaf-node search for a value associated with a key.
+     *
+     * @param pos Position from which the search should start if valid. In case
+     *            of an invalid position, it falls back to a search starting
+     *            with the first position.
+     * @param key Key value to perform search for.
+     *
+     * @return Iterator position associated with the start position of the
+     *         segment containing the key, or end iterator position if the
+     *         search has failed.
+     */
+    const_iterator search(const_iterator pos, key_type key) const;
+
     /**
      * Perform tree search for a value associated with a key.  This method
      * assumes that the tree is valid.  Call is_tree_valid() to find out
@@ -472,7 +443,7 @@ public:
      * @param start_key pointer to a variable where the start key value of the
      *                  segment that contains the key gets stored upon
      *                  successful search.
-     * @param end_key pointer to a varaible where the end key value of the
+     * @param end_key pointer to a variable where the end key value of the
      *                segment that contains the key gets stored upon
      *                successful search.
      * @return a pair of const_iterator corresponding to the start position of
@@ -482,6 +453,18 @@ public:
     std::pair<const_iterator, bool> search_tree(
         key_type key, value_type& value, key_type* start_key = nullptr, key_type* end_key = nullptr) const;
 
+    /**
+     * Perform tree search for a value associated with a key.  The tree must be
+     * valid before performing the search, else the search will fail.
+     *
+     * @param key Key value to perform search for.
+     *
+     * @return Iterator position associated with the start position of the
+     *         segment containing the key, or end iterator position if the
+     *         search has failed.
+     */
+    const_iterator search_tree(key_type key) const;
+
     /**
      * Build a tree of non-leaf nodes based on the values stored in the leaf
      * nodes.  The tree must be valid before you can call the search_tree()
@@ -493,7 +476,19 @@ public:
      * @return true if the tree is valid, otherwise false.  The tree must be
      *         valid before you can call the search_tree() method.
      */
-    bool is_tree_valid() const
+    bool valid_tree() const noexcept
+    {
+        return m_valid_tree;
+    }
+
+    /**
+     * @deprecated Use valid_tree() instead.
+     *
+     * @return true if the tree is valid, otherwise false.  The tree must be
+     *         valid before you can call the search_tree() method.
+     */
+    [[deprecated("Use valid_tree() instead")]]
+    bool is_tree_valid() const noexcept
     {
         return m_valid_tree;
     }
@@ -503,24 +498,24 @@ public:
      * comparing the keys and the values of the leaf nodes only.  Neither the
      * non-leaf nodes nor the validity of the tree is evaluated.
      */
-    bool operator==(const flat_segment_tree<key_type, value_type>& r) const;
+    bool operator==(const flat_segment_tree& other) const noexcept(nothrow_eq_comparable_v);
 
-    bool operator!=(const flat_segment_tree<key_type, value_type>& r) const
+    bool operator!=(const flat_segment_tree& other) const noexcept(nothrow_eq_comparable_v)
     {
-        return !operator==(r);
+        return !operator==(other);
     }
 
-    key_type min_key() const
+    const key_type& min_key() const noexcept
     {
-        return m_left_leaf->value_leaf.key;
+        return m_left_leaf->key;
     }
 
-    key_type max_key() const
+    const key_type& max_key() const noexcept
     {
-        return m_right_leaf->value_leaf.key;
+        return m_right_leaf->key;
     }
 
-    value_type default_value() const
+    const value_type& default_value() const noexcept
     {
         return m_init_val;
     }
@@ -530,14 +525,41 @@ public:
      *
      * @return number of leaf nodes.
      */
-    size_type leaf_size() const;
+    size_type leaf_size() const
+        noexcept(noexcept(st::detail::count_leaf_nodes<size_type>(m_left_leaf.get(), m_right_leaf.get())))
+    {
+        return st::detail::count_leaf_nodes<size_type>(m_left_leaf.get(), m_right_leaf.get());
+    }
 
 #ifdef MDDS_UNIT_TEST
-    nonleaf_node* get_root_node() const
+    const nonleaf_node* get_root_node() const
     {
         return m_root_node;
     }
 
+    struct tree_dumper_traits
+    {
+        using leaf_type = node;
+        using nonleaf_type = nonleaf_node;
+        using tree_type = flat_segment_tree;
+
+        struct to_string
+        {
+            to_string(const tree_type&)
+            {}
+
+            std::string operator()(const leaf_type& leaf) const
+            {
+                return leaf.to_string();
+            }
+
+            std::string operator()(const nonleaf_type& nonleaf) const
+            {
+                return nonleaf.to_string();
+            }
+        };
+    };
+
     void dump_tree() const
     {
         using ::std::cout;
@@ -546,7 +568,7 @@ public:
         if (!m_valid_tree)
             assert(!"attempted to dump an invalid tree!");
 
-        size_t node_count = mdds::__st::tree_dumper<node, nonleaf_node>::dump(m_root_node);
+        size_t node_count = mdds::st::detail::tree_dumper<tree_dumper_traits>::dump(std::cout, *this, m_root_node);
         size_t node_instance_count = node::get_instance_count();
         size_t leaf_count = leaf_size();
 
@@ -567,8 +589,8 @@ public:
         long node_id = 0;
         while (cur_node)
         {
-            cout << "  node " << node_id++ << ": key = " << cur_node->value_leaf.key
-                 << "; value = " << cur_node->value_leaf.value << endl;
+            cout << "  node " << node_id++ << ": key = " << cur_node->key << "; value = " << cur_node->value_leaf.value
+                 << endl;
             cur_node = cur_node->next;
         }
         cout << endl << "  node instance count = " << node::get_instance_count() << endl;
@@ -593,7 +615,7 @@ public:
                     // Position past the right-mode node.  Invalid.
                     return false;
 
-                if (cur_node->value_leaf.key != *itr)
+                if (cur_node->key != *itr)
                     // Key values differ.
                     return false;
 
@@ -617,7 +639,7 @@ public:
                     // Position past the left-mode node.  Invalid.
                     return false;
 
-                if (cur_node->value_leaf.key != *itr)
+                if (cur_node->key != *itr)
                     // Key values differ.
                     return false;
 
@@ -667,11 +689,13 @@ public:
 #endif
 
 private:
-    flat_segment_tree(); // default constructor is not allowed.
+    const_iterator search_by_key_impl(const node* start_pos, key_type key) const;
+
+    const node* search_tree_for_leaf_node(key_type key) const;
 
     void append_new_segment(key_type start_key)
     {
-        if (m_right_leaf->prev->value_leaf.key == start_key)
+        if (m_right_leaf->prev->key == start_key)
         {
             m_right_leaf->prev->value_leaf.value = m_init_val;
             return;
@@ -680,7 +704,7 @@ private:
 #ifdef MDDS_UNIT_TEST
         // The start position must come after the position of the last node
         // before the right-most node.
-        assert(m_right_leaf->prev->value_leaf.key < start_key);
+        assert(m_right_leaf->prev->key < start_key);
 #endif
 
         if (m_right_leaf->prev->value_leaf.value == m_init_val)
@@ -689,12 +713,12 @@ private:
             return;
 
         node_ptr new_node(new node);
-        new_node->value_leaf.key = start_key;
+        new_node->key = start_key;
         new_node->value_leaf.value = m_init_val;
         new_node->prev = m_right_leaf->prev;
         new_node->next = m_right_leaf;
         m_right_leaf->prev->next = new_node;
-        m_right_leaf->prev = new_node;
+        m_right_leaf->prev = std::move(new_node);
         m_valid_tree = false;
     }
 
@@ -702,14 +726,24 @@ private:
         key_type start_key, key_type end_key, value_type val, bool forward);
 
     ::std::pair<const_iterator, bool> insert_to_pos(
-        node_ptr& start_pos, key_type start_key, key_type end_key, value_type val);
+        node_ptr start_pos, key_type start_key, key_type end_key, value_type val);
 
     ::std::pair<const_iterator, bool> search_impl(
         const node* pos, key_type key, value_type& value, key_type* start_key, key_type* end_key) const;
 
-    const node* get_insertion_pos_leaf_reverse(key_type key, const node* start_pos) const;
+    const node* get_insertion_pos_leaf_reverse(const key_type& key, const node* start_pos) const;
 
-    const node* get_insertion_pos_leaf(key_type key, const node* start_pos) const;
+    /**
+     * Find a node with the largest value whose key equals or less than a
+     * specified key, starting with a specific node.
+     *
+     * @pre The caller must ensure that the key equals or greater than the min
+     *      key.
+     *
+     * @note If the key exceeds the max key value, it will return
+     *       <code>nullptr</code>.
+     */
+    const node* get_insertion_pos_leaf(const key_type& key, const node* start_pos) const;
 
     static void shift_leaf_key_left(node_ptr& begin_node, node_ptr& end_node, key_type shift_value)
     {
@@ -717,18 +751,18 @@ private:
         node* end_node_p = end_node.get();
         while (cur_node_p != end_node_p)
         {
-            cur_node_p->value_leaf.key -= shift_value;
+            cur_node_p->key -= shift_value;
             cur_node_p = cur_node_p->next.get();
         }
     }
 
     static void shift_leaf_key_right(node_ptr& cur_node, node_ptr& end_node, key_type shift_value)
     {
-        key_type end_node_key = end_node->value_leaf.key;
+        key_type end_node_key = end_node->key;
         while (cur_node.get() != end_node.get())
         {
-            cur_node->value_leaf.key += shift_value;
-            if (cur_node->value_leaf.key < end_node_key)
+            cur_node->key += shift_value;
+            if (cur_node->key < end_node_key)
             {
                 // The node is still in-bound.  Keep shifting.
                 cur_node = cur_node->next;
@@ -744,10 +778,10 @@ private:
             {
                 node_ptr next_node = cur_node->next;
                 disconnect_all_nodes(cur_node.get());
-                cur_node = next_node;
+                cur_node = std::move(next_node);
             }
             last_node->next = end_node;
-            end_node->prev = last_node;
+            end_node->prev = std::move(last_node);
             return;
         }
     }
@@ -766,7 +800,7 @@ private:
 private:
     std::vector<nonleaf_node> m_nonleaf_node_pool;
 
-    nonleaf_node* m_root_node;
+    const nonleaf_node* m_root_node;
     node_ptr m_left_leaf;
     node_ptr m_right_leaf;
     value_type m_init_val;
@@ -774,7 +808,8 @@ private:
 };
 
 template<typename Key, typename Value>
-void swap(flat_segment_tree<Key, Value>& left, flat_segment_tree<Key, Value>& right)
+void swap(flat_segment_tree<Key, Value>& left, flat_segment_tree<Key, Value>& right) noexcept(
+    std::is_nothrow_swappable_v<flat_segment_tree<Key, Value>>)
 {
     left.swap(right);
 }
@@ -783,6 +818,4 @@ void swap(flat_segment_tree<Key, Value>& left, flat_segment_tree<Key, Value>& ri
 
 #include "flat_segment_tree_def.inl"
 
-#endif
-
 /* vim:set shiftwidth=4 softtabstop=4 expandtab: */
